/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * IVR API Header File
 * 
 * Copyright (C) 2005, Anthony Minessale II.
 *
 * Anthony Minessale <anthmct@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 * Internal information and typedefs.
 * as little information about the objects are made available to the 
 * user of the API (See "API Calls" below)
 ******************************************************************************************************/
#ifndef _ASTERISK_IVR_API_H
#define _ASTERISK_IVR_API_H
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <asterisk/channel.h>
#include <asterisk/musiconhold.h>
#include <asterisk/pbx.h>
#include <asterisk/file.h>
#include <asterisk/app.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
typedef enum {
	IVR_MENU_FLAG_FALLTOMAIN = (1 << 0),
} ast_ivr_menu_flags;
/* Actions are either set in ast_ivr_menu_bind_function or returned by a callback */
typedef enum {
	IVR_ACTION_DIE, 		/* Exit the menu.                  */
	IVR_ACTION_EXECMENU,	/* Goto another menu in the stack. */
	IVR_ACTION_EXECAPP,		/* Execute an application.         */
	IVR_ACTION_PLAYSOUND,	/* Play a sound.                   */
	IVR_ACTION_BACK,		/* Go back 1 menu.                 */
	IVR_ACTION_TOMAIN,		/* Go back to the top level menu.  */
	IVR_ACTION_TRANSFER,	/* Transfer caller to another ext. */
} ast_ivr_action_t;
struct ast_ivr_menu;
typedef ast_ivr_action_t ast_ivr_menu_action_function_t(struct ast_ivr_menu *, char *, size_t, void *);
typedef struct ast_ivr_menu ast_ivr_menu_t;
typedef struct ast_ivr_menu_action ast_ivr_menu_action_t;
/******************************************************************************************************/

/* Usable API Calls
 *
 * ast_ivr_menu_init: Create a new menu object.
 *
 * This function will initilize and return a menu ready to add bindings to (see ast_ivr_menu_bind_* below).
 *
 * main					- The top level menu, (NULL if this is the top level one).
 * name					- A pointer to the name of this menu.
 * greeting_sound 		- Optional pointer to a main sound (press 1 for this 2 for that).
 * short_greeting_sound - Optional pointer to a shorter main sound for subsequent loops.
 * invalid_sound		- Optional pointer to a sound to play after invalid input.
 * timeout				- A number of milliseconds to pause before looping.
 * max_failures			- Maximum number of failures to withstand before hangingup
 *						  This resets everytime you enter the menu.
 * inlen				- The max len of the input (1 for 1 digit menu opts, 2 for 2 digit etc.)
 * flags				- Optional parameters to set to influence the menu behaviour.
 */
ast_ivr_menu_t *ast_ivr_menu_init(ast_ivr_menu_t *main,
								  char *name, 
								  char *greeting_sound, 
								  char *short_greeting_sound,
								  char *invalid_sound, 
								  int timeout,
								  int max_failures, 
								  int inlen,
								  unsigned int flags);

/* ast_ivr_menu_bind_action: Bind a keystroke to an action.
 *
 * menu 				- The menu obj you wish to bind to.
 * ivr_action			- ast_ivr_action_t enum of what you want to do.
 * arg					- Optional (sometimes necessary) string arguement.
 * bind					- KeyStrokes to bind the action to.
 */
void ast_ivr_menu_bind_action(ast_ivr_menu_t *menu, ast_ivr_action_t ivr_action, char *arg, char *bind);


/* ast_ivr_menu_bind_function: Bind a keystroke to a callback function.
 *
 * menu 				- The menu obj you wish to bind to.
 * function				- The function to call [int proto(struct ast_ivr_menu *, char *, size_t, void *)]
 * bind					- KeyStrokes to bind the action to.
 *
 * The function is passed a buffer to fill in with any required argument data.
 * The function is also passed an optional void pointer to an object set upon menu execution. (think threads)
 * The function returns an ast_ivr_action_t enum of what you want to do. and looks to your buffer for args.
 */
void ast_ivr_menu_bind_function(ast_ivr_menu_t *menu, ast_ivr_menu_action_function_t *function, char *bind);


/* ast_ivr_menu_execute: Execute a menu.
 * 
 * This function should be called on a prepared stack of menus to begin execution.
 *
 * chan					- The channel running the menu.
 * stack				- The top-level menu object (the first one you created.)
 * name					- A pointer to the name of the menu.
 * obj					- A void pointer to an object you want to make avaliable to your callback functions
 *				  		  that you may have binded with ast_ivr_menu_bind_function.
 */
int ast_ivr_menu_execute(struct ast_channel *chan, ast_ivr_menu_t *stack, char *name, void *obj);

/* ast_ivr_menu_free: free a menu object.
 * 
 * This function destroys a menu, freeing any memory it may be using.
 *
 * menu					- The menu you wish to destroy.
 *
 */
void ast_ivr_menu_free(ast_ivr_menu_t *menu);


/* ast_ivr_menu_free_stack: free a stack of menu objects.
 * 
 * This function destroys a stack of menus, freeing any memory they may be using.
 *
 * stack				- The top level menu you wish to destroy.
 *
 */
void ast_ivr_menu_free_stack(ast_ivr_menu_t *stack);


/* ast_ivr_menu_set_flags: ast_ivr_menu_clear_flags: ast_ivr_menu_test_flags: Manipulate Flags
 *
 * These functions alter the behaviour of the menus via flags.
 *
 * menu					- The menu to manipulate.
 * flags				- The logical list of flags.
 *
 */
int ast_ivr_menu_set_flags(ast_ivr_menu_t *menu, unsigned int flags);
int ast_ivr_menu_clear_flags(ast_ivr_menu_t *menu, unsigned int flags);
int ast_ivr_menu_test_flags(ast_ivr_menu_t *menu, unsigned int flags);


#endif
